clear all;
close all
clc

fpath   = '/net/wave3/ganeshgopal/WAVE5_DATA/GOM_OI_24DEG/FORCINGS/';
hycompath = '/net/wave3/ganeshgopal/WAVE5_DATA/GOM_OI_24DEG/hycom_15d/';

%% Grid - Mask of MITGCM model
eval(['load ' fpath 'FMT.mat']);
eval(['load ' fpath 'GRID.mat']);

%% X = 101 (LON), Y = 98(LAT), Z = 33(DEPTH), 121(TIME) 
nx1 =  101; ny1 = 98; nz1 = 33; nt1 = 121;

%% READING FROM BINARY FILES
time_hycom = rdslice([hycompath 'time_hycom.bin'],[1 nt1],1,fmt,Ieee);
ntottime = length(time_hycom);    
date = ['2004_2008_GOM_OI_' datestr(time_hycom(1),3) datestr(time_hycom(1),7)];

nPx = 1;
nPy = 1;
rho = 1024;%% density of water
cp  = 4000;%% CP constant for TEMP tranport

%% Lats and longs where to compute HYCOM transports.
%% this is with respect to MITGCM model boundaries
%% dlat = dlon = 0.0208 degrees for GOM 1-48 DEG
%% (NORTH, SOUTH AND EAST WEST Open Boundaries
ddd = 0.04;
tablat = [(yc(1) - ddd) (yc(end) + ddd)];%% SOUTH >>> NORTH BOUND
tablon = [(xc(1) - ddd) (xc(end) + ddd)];%%% WEST >> EAST BOUND

%% this mask file is diff from pmask.bin
%% this is obtained by spininig the model(actual model mask, which is
%% created by the MITGCM model
%% Grid - Mask

eval(['load ' fpath 'FMT.mat']);
eval(['load ' fpath 'GRID.mat']);

msk  = rdmds([fpath 'MASK/hFacC']);
msku = rdmds([fpath 'MASK/hFacW']);
mskv = rdmds([fpath 'MASK/hFacS']);

msk(msk~=0) = 1;
msku(msku~=0) = 1;
mskv(mskv~=0) = 1;

dxv = rdmds([fpath 'MASK/dXV']);
dxc = rdmds([fpath 'MASK/dXC']);
dyu = rdmds([fpath 'MASK/dYU']);
dyc = rdmds([fpath 'MASK/dYC']);

eval(['load ' fpath 'MASK/rF_rC']);
rf  = rF_rC(:,1);
rc  = rF_rC(:,2);
drf = rF_rC(:,3);
drc = rF_rC(:,4);

vn  = zeros(nxc,nzc,ntottime);
vs  = zeros(nxc,nzc,ntottime);
uw  = zeros(nyc,nzc,ntottime);
ue  = zeros(nyc,nzc,ntottime);
htn = zeros(nxc,nzc,ntottime);
hts = zeros(nxc,nzc,ntottime);
htw = zeros(nyc,nzc,ntottime);
hte = zeros(nyc,nzc,ntottime);

%% Position of the open boundaries (as defined in the nested model)
%% South.
obcsS = 'o';
if strcmp(obcsS,'o') == 1
  OB_Jsouth = [ones(nxc,1)]';
end

%% West.
obcsW = 'o';
if strcmp(obcsW,'o') == 1
  OB_Iwest = [ones(96,1);zeros(36,1)]';
end

%% East.
obcsE = 'o';
if strcmp(obcsE,'o') == 1
  OB_Ieast = [nxc*ones(110,1);zeros(22,1)]';
end


%% HYCOM fluxes %%% RAW ANALYSIS 
%% Meridional transport.
eval(['load ' fpath 'HYCOM_HF_MT_' date]);
vhycomtotm = eval(['HYCOM_HF_MT_' date]);
hthycoms = vhycomtotm(:,1);
vthycoms = vhycomtotm(:,3);

%% Zonal transport.
eval(['load ' fpath 'HYCOM_HF_ZT_' date]);
uhycomtotz = eval(['HYCOM_HF_ZT_' date]);
hthycomw = uhycomtotz(:,1);
hthycome = uhycomtotz(:,2);
uthycomw = uhycomtotz(:,3);
uthycome = uhycomtotz(:,4);

%% Balance transport: Take out 1/? of the total transport from each
%% of the three boundaries.
totransp =  - vthycoms - uthycomw + uthycome;

%% computing hycom alone tranport
totransp_hycom =  - vthycoms + uthycome - uthycomw;
vthycoms1 = vthycoms;
uthycome1 = uthycome;
uthycomw1 = uthycomw;
hycom_s = mean(vthycoms1);
hycom_e = mean(uthycome1);
hycom_w = mean(uthycomw1);

abstotransp = abs(vthycoms) + abs(uthycomw) + abs(uthycome);
cs = abs(vthycoms./abstotransp);
cw = abs(uthycomw./abstotransp);
ce = abs(uthycome./abstotransp);
vthycoms = vthycoms + cs.*totransp;
uthycomw = uthycomw + cw.*totransp;
uthycome = uthycome - ce.*totransp;

%% Open boundaries files

%% read interpolated fields at open boundaries

%% South
if strcmp(obcsS,'o') == 1
  obsv = rdda([fpath 'Rs_VobcsS_' date '_nPy' num2str(nPy) '.bin'],[nxc nPy nzc ntottime],1,fmt,Ieee);
  obst = rdda([fpath 'Rs_TobcsS_' date '_nPy' num2str(nPy) '.bin'],[nxc nPy nzc ntottime],1,fmt,Ieee);
end

%% East
if strcmp(obcsE,'o') == 1
  obeu = rdda([fpath 'Rs_UobcsE_' date '_nPx' num2str(nPx) '.bin'],[nyc nPx nzc ntottime],1,fmt,Ieee);
  obet = rdda([fpath 'Rs_TobcsE_' date '_nPx' num2str(nPx) '.bin'],[nyc nPx nzc ntottime],1,fmt,Ieee);
end

%% West
if strcmp(obcsW,'o') == 1
  obwu = rdda([fpath 'Rs_UobcsW_' date '_nPx' num2str(nPx) '.bin'],[nyc nPx nzc ntottime],1,fmt,Ieee);
  obwt = rdda([fpath 'Rs_TobcsW_' date '_nPx' num2str(nPx) '.bin'],[nyc nPx nzc ntottime],1,fmt,Ieee);
end


%% Compute the model transport at the boundaries
%% HERE HYCOM IS INTERPOLATED TO MODEL USUNGI INTERP2 (BILINEAR
%% INTERPOLATION), OR INTERP2GAUSS>> AND THE TRANSPORT IS CALCULATED WITH
%% RESPECT TO MITGCM MODEL GRID    

%% Southern transport
if strcmp(obcsS,'o') == 1
surfs = 0;
for ii = 2:nxc-1
  if OB_Jsouth(ii) ~= 0
    ij = OB_Jsouth(ii);
    for ik = 1:nzc
      surfs = surfs + dxv(ii,ij+1)*drf(ik)*mskv(ii,ij+1,ik);
      vs(ii,ik,:)  = obsv(ii,1,ik,:)*dxv(ii,ij+1)*drf(ik)*mskv(ii,ij+1,ik);
      hts(ii,ik,:) = rho*cp*obst(ii,1,ik,:).*obsv(ii,1,ik,:)*dxc(ii,ij)*drf(ik)*mskv(ii,ij+1,ik);
    end
  end
end
vstot  = squeeze(sum(sum(vs)));
htstot = squeeze(sum(sum(hts)));
end
%% Western transport 
if strcmp(obcsW,'o') == 1
surfw = 0;
for ij = 2:nyc-1
  if OB_Iwest(ij) ~= 0
    ii = OB_Iwest(ij);
    for ik = 1:nzc
      surfw = surfw + dyu(ii+1,ij)*drf(ik)*msku(ii+1,ij,ik);
      uw(ij,ik,:)  = obwu(ij,1,ik,:)*dyu(ii+1,ij)*drf(ik)*msku(ii+1,ij,ik);
      htw(ij,ik,:) = rho*cp*obwu(ij,1,ik,:).*obwt(ij,1,ik,:)*dyc(ii,ij)*drf(ik)*msku(ii+1,ij,ik);
    end 
  end
end
uwtot  = squeeze(sum(sum(uw)));
htwtot = squeeze(sum(sum(htw)));
end
%% East Transport
if strcmp(obcsE,'o') == 1
surfe = 0;
for ij = 2:nyc-1
  if OB_Ieast(ij) ~= 0
    ii = OB_Ieast(ij);
    for ik = 1:nzc
      surfe = surfe + dyu(ii,ij)*drf(ik)*msku(ii,ij,ik);
      ue(ij,ik,:)  = obeu(ij,1,ik,:)*dyu(ii,ij)*drf(ik)*msku(ii,ij,ik);
      hte(ij,ik,:) = rho*cp*obeu(ij,1,ik,:).*obet(ij,1,ik,:)*dyc(ii,ij)*drf(ik)*msku(ii,ij,ik);
    end
  end
end
uetot  = squeeze(sum(sum(ue)));
htetot = squeeze(sum(sum(hte)));
end

% ---------------------------------------------
%% ADDING THE DIFFERENCE BETWEEN THE HYCOM RAW AND INTERPOLATED MITGCM 
%% TRANSPORTS TO MITGCM TRANSPORT, NORMALIZED BY THE AREA SLICE
%% --> North : vntotcor = vntot+cor = vthycomon;

%% --> South : vstotcor = vstot+cor = vthycoms;
%% Compute correction
if strcmp(obcsS,'o') == 1
vcors = (vthycoms - vstot)/surfs;
obsvcor = obsv;

for it = 1:ntottime
  for ii = 2:nxc-1
    if OB_Jsouth(ii) ~= 0
      ij = OB_Jsouth(ii);
      for ik = 1:nzc
        obsvcor(ii,:,ik,it) = (obsvcor(ii,:,ik,it) + vcors(it))*mskv(ii,ij+1,ik);
      end
    end
  end
end

%% Compute new transport
for ii = 2:nxc-1
  if OB_Jsouth(ii) ~= 0
    ij = OB_Jsouth(ii);
    for ik = 1:nzc
      vscor(ii,ik,:)  = obsvcor(ii,1,ik,:)*dxv(ii,ij+1)*drf(ik)*mskv(ii,ij+1,ik);
      htscor(ii,ik,:) = rho*cp*obst(ii,1,ik,:).*obsvcor(ii,1,ik,:)*dxc(ii,ij)*drf(ik)*mskv(ii,ij+1,ik);
    end
  end
end
vscortot  = squeeze(sum(sum(vscor)));
htscortot = squeeze(sum(sum(htscor)));
end

%% --> West : uwtotcor = uwtot+cor = uthycomw;

%% Compute correction
if strcmp(obcsW,'o') == 1
ucorw = (uthycomw - uwtot)/surfw;
obwucor = obwu;
for it = 1:ntottime
  for ij = 2:nyc-1
    if OB_Iwest(ij) ~= 0
      ii = OB_Iwest(ij);
      for ik = 1:nzc
        obwucor(ij,:,ik,it) = (obwucor(ij,:,ik,it) + ucorw(it))*msku(ii+1,ij,ik);
      end
    end
  end
end

%% Compute new transport
for ij = 2:nyc-1
  if OB_Iwest(ij) ~= 0
    ii = OB_Iwest(ij);
    for ik = 1:nzc
      uwcor(ij,ik,:)  = obwucor(ij,1,ik,:)*dyu(ii+1,ij)*drf(ik)*msku(ii+1,ij,ik);
      htwcor(ij,ik,:) = rho*cp*obwt(ij,1,ik,:).*obwucor(ij,1,ik,:)*dyc(ii,ij)*drf(ik)*msku(ii+1,ij,ik);
    end
  end
end
uwcortot  = squeeze(sum(sum(uwcor)));
htwcortot = squeeze(sum(sum(htwcor)));
end
%% --> East : uetotcor = uetot+cor = uteccoe;

%% Compute correction
if strcmp(obcsE,'o') == 1
ucore = (uthycome - uetot)/surfe;
obeucor = obeu;
for it = 1:ntottime
  for ij = 2:nyc-1
    if OB_Ieast(ij) ~= 0
      ii = OB_Ieast(ij);
      for ik = 1:nzc
        obeucor(ij,:,ik,it) = (obeucor(ij,:,ik,it) + ucore(it))*msku(ii,ij,ik);
      end
    end
  end
end

%% Compute new transport - East
for ij = 2:nyc-1
  if OB_Ieast(ij) ~= 0
    ii = OB_Ieast(ij);
    for ik = 1:nzc
      uecor(ij,ik,:)  = obeucor(ij,1,ik,:)*dyu(ii,ij)*drf(ik)*msku(ii,ij,ik);
      htecor(ij,ik,:) = rho*cp*obet(ij,1,ik,:).*obeucor(ij,1,ik,:)*dyc(ii,ij)*drf(ik)*msku(ii,ij,ik);
    end
  end
end
uecortot  = squeeze(sum(sum(uecor)));
htecortot = squeeze(sum(sum(htecor)));
end

%% PLOTTING MITGCM MODEL INTERPOLATED TRANSPORT
figure(1)
subplot(1,2,1)
dummy  = squeeze(obwucor(:,1,:,2))';
dummy(dummy == 0) = NaN;
contourf(yc,zc,dummy)
colorbar
set(gca,'FontSize',14);
title(['Zonal Velocity at ' num2str((tablon(1))) '^oE'],'FontSize',14); 
xlabel('Latitude (deg) ','FontSize',14)
ylabel('Depth (m)','FontSize',14)

subplot(1,2,2)
dummy = squeeze(obwt(:,1,:,2))';
dummy(dummy == 0) = NaN;
contourf(yc,zc,dummy)
colorbar
set(gca,'FontSize',14);
title(['Zonal Velocity at ' num2str((tablon(1))) '^oE'],'FontSize',14); 
eval(['print -depsc ' fpath 'MITGCM_INTER_VEL_TEMP_ZON_' date '_' num2str((tablon(1))) 'E']);
xlabel('Latitude (deg) ','FontSize',14)
ylabel('Depth (m)','FontSize',14)

figure(2)
subplot(1,2,1)
dummy = squeeze(obeucor(:,1,:,2))';
dummy(dummy == 0) = NaN;
contourf(yc,zc,dummy)
colorbar
set(gca,'FontSize',14);
title(['Zonal Velocity at ' num2str((tablon(2))) '^oE'],'FontSize',14); 
xlabel('Latitude (deg) ','FontSize',14)
ylabel('Depth (m)','FontSize',14)

subplot(1,2,2)
dummy = squeeze(obet(:,1,:,2))';
dummy(dummy == 0) = NaN;
contourf(yc,zc,dummy)
colorbar
set(gca,'FontSize',14);
title(['Zonal Velocity at ' num2str((tablon(2))) '^oE'],'FontSize',14); 
eval(['print -depsc ' fpath 'MITGCM_INTER_VEL_TEMP_ZON_' date '_' num2str((tablon(2))) 'E']);
xlabel('Latitude (deg) ','FontSize',14)
ylabel('Depth (m)','FontSize',14)

figure(3)
subplot(1,2,1)
dummy = squeeze(obsvcor(:,1,:,2))';
dummy(dummy == 0) = NaN;
contourf(xc,zc,dummy)
colorbar
set(gca,'FontSize',14);
title(['Meridiaonal Velocity at ' num2str((tablat(1))) '^oN'],'FontSize',14); 
xlabel('Longitude (deg) ','FontSize',14)
ylabel('Depth (m)','FontSize',14)


subplot(1,2,2)
dummy = squeeze(obst(:,1,:,2))';
dummy(dummy == 0) = NaN;
contourf(xc,zc,dummy)
colorbar
set(gca,'FontSize',14);
title(['Meridiaonal Velocity at ' num2str((tablat(1))) '^oN'],'FontSize',14); 
eval(['print -depsc ' fpath 'MITGCM_INTER_VEL_TEMP_MER_' date '_' num2str((tablat(1))) 'N']);
xlabel('Longitude (deg) ','FontSize',14)
ylabel('Depth (m)','FontSize',14)



%% All
totransp = - vscortot - uwcortot +uecortot;
figure(10)
plot(totransp)

%% BALACING THE CORRECTED TRANSPORT SO AS TO MAKE THE SUMMATION IS ZERO
%% VOLUME TRANSPORT IS CONSERVED
totransp =  - vscortot - uwcortot + uecortot;
abstotransp = abs(vscortot)+abs(uwcortot) + abs(uecortot);
cs = abs(vscortot./abstotransp);
cw = abs(uwcortot./abstotransp);
ce = abs(uecortot./abstotransp);
vscortot = vscortot + cs.*totransp;
uwcortot = uwcortot + cw.*totransp;
uecortot = uecortot - ce.*totransp;

totransp =  - vscortot - uwcortot + uecortot;



%% Plots

mehts = mean([htscortot htstot hthycoms]);
mehtw = mean([htwcortot htwtot hthycomw]);
mehte = mean([htecortot htetot hthycome]);


mevs = mean([vscortot vstot vthycoms]);
meuw = mean([uwcortot uwtot uthycomw]);
meue = mean([uecortot uetot uthycome]);


mevs2 = num2str((mevs'*100*1e-6)/100);
meuw2 = num2str((meuw'*100*1e-6)/100);
meue2 = num2str((meue'*100*1e-6)/100);


mehts2 = num2str((mehts'*100*1e-15)/100);
mehtw2 = num2str((mehtw'*100*1e-15)/100);
mehte2 = num2str((mehte'*100*1e-15)/100);

%% South
figure
subplot(2,1,1); hold on;
set(gca,'FontSize',14)
h1 = plot([1:ntottime],vscortot*1e-6,'b',[1:ntottime],vstot*1e-6,'r',[1:ntottime],vthycoms*1e-6,'g--','LineWidth',2); grid on;
title(['Meridional Transport (Sv) - ' num2str((tablat(1))) '^oN - means = ' mevs2(1,:) ' ' mevs2(2,:) ' ' mevs2(3,:)])
legend(h1,'Corrected','Non-Corrected','HYCOM');
subplot(2,1,2); hold on; grid on;
set(gca,'FontSize',14)
plot([1:ntottime],htscortot*1e-15,'b',[1:ntottime],htstot*1e-15,'r',[1:ntottime],hthycoms*1e-15,'g','LineWidth',2); grid on;
title(['Meridional Heat Transport (10^{15} W) - ' num2str((tablat(1))) '^oN - means = ' mehts2(1,:) ' ' mehts2(2,:) ' ' mehts2(3,:)])
legend('Corrected','Non-Corrected','HYCOM');
eval(['print -depsc ' fpath 'COR_VEL_TEMP_MER_' date '_' num2str((tablat(1))) 'N']);


%% West
figure
subplot(2,1,1); hold on; 
set(gca,'FontSize',14)
plot([1:ntottime],uwcortot*1e-6,'b',[1:ntottime],uwtot*1e-6,'r',[1:ntottime],uthycomw*1e-6,'g--','LineWidth',2);grid on;
title(['Zonal Transport (Sv) - ' num2str((tablon(1))) '^oE - means = ' meuw2(1,:) ' ' meuw2(2,:) ' ' meuw2(3,:)])
legend('Corrected','Non-Corrected','HYCOM');
subplot(2,1,2); hold on; grid on;
set(gca,'FontSize',14)
plot([1:ntottime],htwcortot*1e-15,'b',[1:ntottime],htwtot*1e-15,'r',[1:ntottime],hthycomw*1e-15,'g--','LineWidth',2);grid on;
title(['Zonal Heat Transport (10^{15} W) - ' num2str((tablon(1))) '^oE - means = ' mehtw2(1,:) ' ' mehtw2(2,:) ' ' mehtw2(3,:)])
legend('Corrected','Non-Corrected','HYCOM');
eval(['print -depsc ' fpath 'COR_VEL_TEMP_ZON_' date '_' num2str((tablon(1))) 'E']);


%% East
figure
subplot(2,1,1); hold on; 
set(gca,'FontSize',14)
plot([1:ntottime],uecortot*1e-6,'b',[1:ntottime],uetot*1e-6,'r',[1:ntottime],uthycome*1e-6,'g--','LineWidth',2);grid on;
title(['Zonal Transport (Sv) - ' num2str((tablon(2))) '^oE - means = ' meue2(1,:) ' ' meue2(2,:) ' ' meue2(3,:)])
legend('Corrected','Non-Corrected','HYCOM');
subplot(2,1,2); hold on; grid on;
set(gca,'FontSize',14)
plot([1:ntottime],htecortot*1e-15,'b',[1:ntottime],htetot*1e-15,'r',[1:ntottime],hthycome*1e-15,'g--','LineWidth',2);grid on;
title(['Zonal Heat Transport (10^{15} W) - ' num2str((tablon(2))) '^oE - means = ' mehte2(1,:) ' ' mehte2(2,:) ' ' mehte2(3,:)])
legend('Corrected','Non-Corrected','HYCOM');
eval(['print -depsc ' fpath 'COR_VEL_TEMP_ZON_' date '_' num2str((tablon(2))) 'E']);


figure
hold on;
set(gca,'FontSize',14); 
plot([1:ntottime],vscortot*1e-6,'r--',[1:ntottime],uwcortot*1e-6,'g--',[1:ntottime],uecortot*1e-6,'m--',[1:ntottime],totransp*1e-6,'k','LineWidth',2)
title(['Transport (Sv) means: S: ' mevs2(1,:) ', W: ' meuw2(1,:) ', E: ' meue2(1,:)])
legend('South','West','East','Total'); grid on;
eval(['print -depsc ' fpath 'COR_VEL_TOTAL_' date '_' num2str((tablat(1))) '-' num2str((tablat(2))) '-' num2str((tablon(1))) '-' num2str((tablon(2)))]);


%% All
totransp    = - vstot + uetot - uwtot;
totranspcor = - vscortot - uwcortot + uecortot;


figure
subplot(2,1,1); 
hold on; grid on;
set(gca,'FontSize',14); plot([1:ntottime],vstot*1e-6,'r--',[1:ntottime],uetot*1e-6,'g--',[1:ntottime],uwtot*1e-6,'c--',[1:ntottime],totransp*1e-6,'k','LineWidth',2)
title(['SV Before Correction, Means: S: ' num2str(1e-6*mevs(2)) ', E: ' num2str(1e-6*meue(2)) ', W: ' num2str(1e-6*meuw(2))])
legend('South','East','West','Total');
subplot(2,1,2); 
hold on; grid on;
set(gca,'FontSize',14); plot([1:ntottime],vscortot*1e-6,'r--',[1:ntottime],uecortot*1e-6,'g--',[1:ntottime],uwcortot*1e-6,'c--',[1:ntottime],totranspcor*1e-6,'k','LineWidth',2)
title(['Sv After Correction, Means: S: ' num2str(1e-6*mevs(1)) ', E: ' num2str(1e-6*meue(1)) ', W: ' num2str(1e-6*meuw(1))])
legend('South','East','West','Total');
eval(['print -depsc ' fpath 'obcall_transp' date ]);

figure
set(gca,'FontSize',14); plot([1:ntottime],vthycoms1*1e-6,'r--',[1:ntottime],uthycome1*1e-6,'g--',[1:ntottime],uthycomw1*1e-6,'c--',[1:ntottime],totransp_hycom*1e-6,'k','LineWidth',2)
grid on
title(['SV HYCOM, Means:S: ' num2str(1e-6*hycom_s) ', E: ' num2str(1e-6*hycom_e) ', W: ' num2str(1e-6*hycom_w)])
legend('South','East','West','Total');


eval(['print -depsc ' fpath 'HYCOM_transp_' date ]);


%% Write output
  
%% South
if strcmp(obcsS,'o') == 1
  wrslice([fpath 'Rs_VobcsS_' date '_nPy' num2str(nPy) '_c1.bin' ],obsvcor,1,fmt,Ieee);
end
 
%% West
if strcmp(obcsW,'o') == 1
  wrslice([fpath 'Rs_UobcsW_' date '_nPx' num2str(nPy) '_c1.bin' ],obwucor,1,fmt,Ieee);
end 


%% East
if strcmp(obcsE,'o') == 1
  wrslice([fpath 'Rs_UobcsE_' date '_nPx' num2str(nPy) '_c1.bin' ],obeucor,1,fmt,Ieee);
end 

