%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% TO CALCULATE VOLUME TRANSPORT AND HEAT TRANSPORT ACCROSS OPEN BOUNDARIES
%% FROM GLOBAL MODEL (HYCOM MODEL USED)
%% modified for HYCOM By ganesh on 121508 for TAIWAN GRID
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Cleaning
clear all
close all
clc

%% My model paths and grid.
fpath   = '/net/wave3/ganeshgopal/WAVE5_DATA/GOM_OI_24DEG/FORCINGS/';
hycompath = '/net/wave3/ganeshgopal/WAVE5_DATA/GOM_OI_24DEG/hycom_15d/';

%% LOADING MITGCM MODEL GRID AND DETAILS
eval(['load ' fpath 'FMT.mat']);
eval(['load ' fpath 'GRID.mat']);

%% X = 101 (LON), Y = 98(LAT), Z = 33(DEPTH), 121(TIME) 
nx1 =  101; ny1 = 98; nz1 = 33; nt1 = 121;
lon_hycom = rdslice([hycompath 'lon_hycom.bin'],[nx1 1],1,fmt,Ieee);
lat_hycom = rdslice([hycompath 'lat_hycom.bin'],[ny1 1],1,fmt,Ieee);
z_hycom = rdslice([hycompath 'depth_hycom.bin'],[nz1 1],1,fmt,Ieee);
time_hycom = rdslice([hycompath 'time_hycom.bin'],[1 nt1],1,fmt,Ieee);
uhycom = rdslice([hycompath 'uhycom.bin'],[nx1 ny1 nz1 nt1],1,fmt,Ieee);
vhycom = rdslice([hycompath 'vhycom.bin'],[nx1 ny1 nz1 nt1],1,fmt,Ieee);
thycom = rdslice([hycompath 'thycom.bin'],[nx1 ny1 nz1 nt1],1,fmt,Ieee);

date = ['2004_2008_GOM_OI_' datestr(time_hycom(1),3) datestr(time_hycom(1),7)];

ntottime = nt1;    

%% HYCOM GRID INFO
y = lat_hycom;
x = lon_hycom;
dz = diff(-z_hycom);
z = cumsum(dz)-dz/2;
%% ADDING AN ADDITIONAL BOTTOM LAYER,AS HYCOM STARTS WITH '0' 
dz = [dz; dz(end)];
%% HYCOM solution outputs.
hycomtfile = thycom;
hycomufile = uhycom;
hycomvfile = vhycom;
ntottime = length(time_hycom);

%% GETTING SIZE OF HYCOM GRID, a = nx1, b = ny1, c = nz1 of HYCOM 
a = nx1; b = ny1; c = nz1;

%% Lats and longs where to compute HYCOM transports.
%% this is with respect to MITGCM model boundaries
%% dlat = dlon = 0.0208 degrees for GOM 1-48 DEG
%% (NORTH, SOUTH AND EAST WEST Open Boundaries
ddd = 0.04;
tablat = [(yc(1) - ddd) (yc(end) + ddd)];%% SOUTH >>> NORTH BOUND
tablon = [(xc(1) - ddd) (xc(end) + ddd)];%%% WEST >> EAST BOUND

%% ###################################################################
%% Loop over the zonal boundaries;
%% ###################################################################
DX = diff(x);%% getting x increments
DY = diff(y);%% getting y increments
dx = [DX; DX(end)].*111111; %% converting into meters (1 deg = 111111 m)
dy = [DY; DY(end)].*111111;
dV  = zeros(a,b,c);
dVT = zeros(a,b,c);
dA  = zeros(a,b,c);
%% computing total VOLUME of hycom grid
for i = 1:a 
  for j = 1:b 
    for k = 1:c
        %% dx(long) is corrected for each dy (lat) by cos(lat) factor
        dV(i,j,k) = dx(i)*cos(y(j)*pi/180)*dy(j)*dz(k);      
    end
  end
end
%%  area SLICES are computed along Longitudes
for i = 1:a
  for j = 1:b 
    for k = 1:c
        %% dx(long) is corrected for each dy (lat) by cos(lat) factor
        dA(i,j,k) = dx(i)*cos(y(j)*pi/180)*dz(k); 
    end
  end
end     
%% SURFACE AREA
for i = 1:a
  for j = 1:b
      %% dx(long) is corrected for each dy (lat) by cos(lat) factor
      AS(i,j) = dx(i)*cos(y(j)*pi/180)*dy(j); 
  end
end 

%% TO GET LAND MANSK OF HYCOM (vel file)
%% missing values in HYCOM are assigned to '0' during extraction
ssh = squeeze(hycomvfile(:,:,:,1));
for k = 1:c
  [xx,yy] = find(squeeze(ssh(:,:,k)) == 0);
  nn = length(xx);
  for ii = 1:nn
    dA(xx(ii),yy(ii),k) = 0;
  end
end

dVT = dV;%%duiplicating for volume tranport

%% TO GET LAND MANSK OF HYCOM (temp file)
%% missing values in HYCOM are assigned to '0' during extraction

ssh = squeeze(hycomtfile(:,:,:,1));
for k = 1:c
  [xx,yy] = find(squeeze(ssh(:,:,k)) == 0);
  nn = length(xx);
  for ii = 1:nn
    dVT(xx(ii),yy(ii),k)=0;    
  end
end


%% Loop over Longitudes to get meridiaonal tranport 
%% getting MITGCM model lats to fix north and south boundaries
nl = length(tablat);

for ip = 1:nl
    phi1 = tablat(ip);    
    lam1 = (xc(1) - ddd);
    lam2 = (xc(end) + ddd);
  
  %% identifying first South and then North extremes
  i1 = find(x-lam1 > 0.0001);
  i2 = find(x-lam2 > 0.0001);
  j1 = find(y-phi1 < 0.0001);
  j1 = j1(end);
  
 %% plotting to check the HYCOM raw analysis
 figure(ip)
 orient tall
 subplot(1,2,1); 
 dummy = squeeze(hycomvfile(i1:i2,j1,:,2))';
 dummy(dummy == 0) = NaN;
 contourf(x(i1:i2),z_hycom',dummy) 
 colorbar
 set(gca,'FontSize',14);
 title(['Meridional Velocity at ' num2str((tablat(ip))) '^oN'],'FontSize',14); 
 xlabel('Longitude (deg) ','FontSize',14)
 ylabel('Depth (m)','FontSize',14)
 
 subplot(1,2,2)
 dummy = squeeze(hycomtfile(i1:i2,j1,:,2))';
 dummy(dummy == 0) = NaN;
 contourf(x(i1:i2),z_hycom',dummy)
 colorbar
 set(gca,'FontSize',14);
 title(['Meridional Temp at ' num2str((tablat(ip))) '^oN'],'FontSize',14); 

 eval(['print -depsc ' fpath 'HYCOM_VEL_TEMP_MER_' date '_' num2str((tablat(ip))) 'N']);

 xlabel('Longitude (deg) ','FontSize',14)
 ylabel('Depth (m)','FontSize',14)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  for l = 1:ntottime
    %% Set T as temp
    T = zeros(a,b,c);
    ssh = squeeze(hycomtfile(:,:,:,l));
    %% MAKE VOLUME WEIGHTED Averaged
    ssh = ssh.*dVT;
    for i = i1:i2
      for k = 1:c
        j = j1-1; %% getting adjacent y
        if((dVT(i,j,k)+dVT(i,j+1,k))>0) 
          T(i,j+1,k) = (ssh(i,j,k)+ssh(i,j+1,k))/(dVT(i,j,k)+dVT(i,j+1,k));
        end
        if(ssh(i,j,k)==0 & dVT(i,j+1,k)>0)
          T(i,j+1,k) =  (ssh(i,j+1,k))/(dVT(i,j+1,k));
        end
        if(ssh(i,j+1,k)==0 & dVT(i,j,k)>0)
          T(i,j+1,k) =  (ssh(i,j,k))/(dVT(i,j,k));
        end
      end
    end 
    %% Set V. as Velocity
    V = squeeze(hycomvfile(:,:,:,l));
    %% Compute transports.
    VT(l,ip) = 1024*4000*sum(sum(T(i1:i2,j1,:).*V(i1:i2,j1,:).*dA(i1:i2,j1,:))); %% TEMP TRANPOSRT
    Vtrans(l,ip) = sum(sum(V(i1:i2,j1,:).*dA(i1:i2,j1,:)));%% Volmue Transport
  end
  
end


%% Plotting the transports
for ip = 1:nl
  figure
  orient tall
  subplot(2,1,1); 
  set(gca,'FontSize',14);
  plot([1:ntottime],VT(:,ip)*1e-15,'b','LineWidth',2);
  grid on;
  title(['Meridional Heat Transport (x 10^{15} W) - Mean = ' ...
	num2str(mean(VT(:,ip))*1e-15)],'FontSize',14);
  subplot(2,1,2); 
  set(gca,'FontSize',14);
  plot([1:ntottime],Vtrans(:,ip)*1e-6,'b','LineWidth',2);
  grid on;
  title(['Meridional Transport (Sv) - Mean = ' ...
	num2str(mean(Vtrans(:,ip))*1e-6)],'FontSize',14);
  suptitle(['HYCOM - ' num2str((tablat(ip))) '^oN'],14);
  eval(['print -depsc ' fpath 'HYCOM_HF_MT_' date '_' num2str((tablat(ip))) 'N']);

end

%% saving for volume adjusment in MITgcm later
vtot = [VT Vtrans];
eval(['save ' fpath 'HYCOM_HF_MT_' date ' vtot -ascii']);

%% ###################################################################
%% Loop over the meridional boundaries;
%% ###################################################################
%% getting dy along HYCOM latitude
dy = [DY; DY(end)].*ones(b,1)*111111; 

%% COMPUTING SLICE AREAS ALONG LATITUDE
for i = 1:a
  for j = 1:b
    for k = 1:c
      dA(i,j,k) = dy(j)*dz(k);  
    end
  end
end     

%% MASKING HYCOM VALUES 
%% missing values in HYCOM are assigned to '0' during extraction
ssh = squeeze(hycomufile(:,:,:,1));
for k = 1:c
  [xx,yy] = find(ssh(:,:,k) == 0);
  nn = length(xx);
  for ii = 1:nn
    dA(xx(ii),yy(ii),k) = 0;
  end
end


%% Loop over Latitudes to get zonal tranport 
%% getting MITGCM model longs to fix west boundary, already east is land
%% closed boundary for san diego model

nl = length(tablon);

for ip = 1:nl
  phi1 = tablon(ip);
  lom1 = (yc(1) - ddd);
  lom2 = (yc(end) + ddd);

 %% identifying west extreme 
  j1 = find(y - lom1 > 0.0001);
  j2 = find(y - lom2 > 0.0001);
  i1 = find(x - phi1 < 0.0001);
  i1 = i1(end);
  
 %% plotting to check the HYCOM raw analysis
 figure
 orient tall
 subplot(1,2,1); 
 dummy = squeeze(hycomufile(i1,j1:j2,:,2))';
 dummy(dummy == 0) = NaN;
 contourf(y(j1:j2),z_hycom',dummy)
 colorbar
 set(gca,'FontSize',14);
 title(['Zonal Velocity at ' num2str((tablon(ip))) '^oE'],'FontSize',14);  
 xlabel('Latitude (deg) ','FontSize',14)
 ylabel('Depth (m)','FontSize',14)
 
 subplot(1,2,2)
 dummy = squeeze(hycomtfile(i1,j1:j2,:,2))';
 dummy(dummy == 0) = NaN;
 contourf(y(j1:j2),z_hycom',dummy)
 colorbar
 set(gca,'FontSize',14);
 title(['Zonal Temp at ' num2str((tablon(ip))) '^oE'],'FontSize',14);
eval(['print -depsc ' fpath 'HYCOM_VEL_TEMP_ZON_' date '_' num2str((tablon(ip))) 'E']);
 xlabel('Latitude (deg) ','FontSize',14)
 ylabel('Depth (m)','FontSize',14)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  for l = 1:ntottime
    %% Set T.for temp
    %% MAKE VOLUME WEIGHTED Averaged
    T = zeros(a,b,c);
    ssh = squeeze(hycomtfile(:,:,:,l));
    ssh = ssh.*dVT;
    for j = j1:j2
      for k = 1:c
        i = i1-1;
        if((dVT(i,j,k)+dVT(i+1,j,k))>0) 
          T(i+1,j,k) = (ssh(i,j,k)+ssh(i+1,j,k))/(dVT(i,j,k)+dVT(i+1,j,k));
        end
        if(ssh(i,j,k)==0 & dVT(i+1,j,k)>0)
          T(i+1,j,k) =  (ssh(i+1,j,k))/(dVT(i+1,j,k));
        end
        if(ssh(i+1,j,k)==0 & dVT(i,j,k)>0)
          T(i+1,j,k) =  (ssh(i,j,k))/(dVT(i,j,k));
        end
      end
    end
    %% Set U.for velocity
    U = squeeze(hycomufile(:,:,:,l));;
    %% Compute transports.
    UT(l,ip) = 1024*4000*sum(sum(T(i1,j1:j2,:).*U(i1,j1:j2,:).*dA(i1,j1:j2,:)));%% TEMP TRSNPORT
    Utrans(l,ip) = sum(sum(U(i1,j1:j2,:).*dA(i1,j1:j2,:)));%% VOLUME TRANSPORT
  end

end


%% Plotting the Zonal Trnaports

for ip = 1:nl
  figure
  orient tall
  subplot(2,1,1); 
  set(gca,'FontSize',14);
  plot([1:ntottime],UT(:,ip)*1e-15,'b','LineWidth',2);
  grid on;
  title(['Zonal Heat Transport (x 10^{15} W) - Mean = ' ...
	num2str(mean(UT(:,ip))*1e-15)],'FontSize',14);
  subplot(2,1,2);
  grid on;
  set(gca,'FontSize',14);
  plot([1:ntottime],Utrans(:,ip)*1e-6,'b','LineWidth',2);
  grid on;
  title(['Zonal Transport (Sv) - Mean = ' ...
    num2str(mean(Utrans(:,ip))*1e-6)],'FontSize',14);
  suptitle(['HYCOM - ' num2str((tablon(ip))) '^oE'],14);
  eval(['print -depsc ' fpath 'HYCOM_HF_ZT_' date '_' num2str((tablon(ip))) 'E']);

end
%% SAVING ZONAL TRANSPORT FOR MITGCM MODEL TRANSPORT CORRECTION
utot = [UT Utrans];
eval(['save ' fpath 'HYCOM_HF_ZT_' date ' utot -ascii']);
totransp = Vtrans(:,2) - Vtrans(:,1) - Utrans(:,1);
