%% Extracts HYCOM initial conditions for GULF OF MEXICO 1-48 deg model
clear all
close all
clc
%% My model paths.
fpath   = '/net/wave3/ganeshgopal/WAVE5_DATA/GOM_OI_24DEG/FORCINGS/';
hycompath = '/net/wave3/ganeshgopal/WAVE5_DATA/GOM_OI_24DEG/hycom_15d/';

%% Grid of the nested model
eval(['load ' fpath 'FMT.mat']);
eval(['load ' fpath 'GRID.mat']);

%% X = 101 (LON), Y = 98(LAT), Z = 33(DEPTH), 121(TIME) 
nx1 =  101; ny1 = 98; nz1 = 33; nt1 = 121;
lon_hycom = rdslice([hycompath 'lon_hycom.bin'],[nx1 1],1,fmt,Ieee);
lat_hycom = rdslice([hycompath 'lat_hycom.bin'],[ny1 1],1,fmt,Ieee);
z_hycom = rdslice([hycompath 'depth_hycom.bin'],[nz1 1],1,fmt,Ieee);
time_hycom = rdslice([hycompath 'time_hycom.bin'],[1 nt1],1,fmt,Ieee);
uhycom = rdslice([hycompath 'uhycom.bin'],[nx1 ny1 nz1 nt1],1,fmt,Ieee);
vhycom = rdslice([hycompath 'vhycom.bin'],[nx1 ny1 nz1 nt1],1,fmt,Ieee);
thycom = rdslice([hycompath 'thycom.bin'],[nx1 ny1 nz1 nt1],1,fmt,Ieee);
shycom = rdslice([hycompath 'shycom.bin'],[nx1 ny1 nz1 nt1],1,fmt,Ieee);

date = ['2004_2008_GOM_OI_' datestr(time_hycom(1),3) datestr(time_hycom(1),7)];

ntottime = nt1;   
nPx = 1;
nPy = 1;

%% Position of the open boundaries (as defined in the nested model)

%% South.
obcsS = 'o';
if strcmp(obcsS,'o') == 1
  OB_Jsouth = [ones(nxc,1)]';
end

%% West.
obcsW = 'o';
if strcmp(obcsW,'o') == 1
  OB_Iwest = [ones(96,1);zeros(36,1)]';
end

%% East.
obcsE = 'o';
if strcmp(obcsE,'o') == 1
  OB_Ieast = [nxc*ones(110,1);zeros(22,1)]';
end

%% Initialization of the arrays

if strcmp(obcsS,'o') == 1
  TobcsS = zeros(nxc,nPy,nzc,ntottime);
  SobcsS = zeros(nxc,nPy,nzc,ntottime);
  UobcsS = zeros(nxc,nPy,nzc,ntottime);
  VobcsS = zeros(nxc,nPy,nzc,ntottime);    
end

if strcmp(obcsE,'o') == 1
  TobcsE = zeros(nyc,nPx,nzc,ntottime);
  SobcsE = zeros(nyc,nPx,nzc,ntottime);
  UobcsE = zeros(nyc,nPx,nzc,ntottime);
  VobcsE = zeros(nyc,nPx,nzc,ntottime);    
end

if strcmp(obcsW,'o') == 1
  TobcsW = zeros(nyc,nPx,nzc,ntottime);
  SobcsW = zeros(nyc,nPx,nzc,ntottime);
  UobcsW = zeros(nyc,nPx,nzc,ntottime);
  VobcsW = zeros(nyc,nPx,nzc,ntottime);    
end


%% Loop on the time steps
indt = 0;
report('Time = xxxxx')

for it = 1:ntottime
  report('\b\b\b\b\b%5i',it) 
  indt = indt+1;  
  [Tm,xc,yc,zc] = exinthycomobcs(fpath,'T',thycom(:,:,:,it),time_hycom(it),lon_hycom,lat_hycom,z_hycom,xc,yc,xf,yf,zc,fmt,Ieee);
  [Sm,xc,yc,zc] = exinthycomobcs(fpath,'S',shycom(:,:,:,it),time_hycom(it),lon_hycom,lat_hycom,z_hycom,xc,yc,xf,yf,zc,fmt,Ieee);
  [Um,xc,yc,zc] = exinthycomobcs(fpath,'U',uhycom(:,:,:,it),time_hycom(it),lon_hycom,lat_hycom,z_hycom,xc,yc,xf,yf,zc,fmt,Ieee);
  [Vm,xc,yc,zc] = exinthycomobcs(fpath,'V',vhycom(:,:,:,it),time_hycom(it),lon_hycom,lat_hycom,z_hycom,xc,yc,xf,yf,zc,fmt,Ieee);
  nzc = length(zc);

  Tin = reshape(Tm,nxc,nyc,nzc);
  Sin = reshape(Sm,nxc,nyc,nzc);
  Uin = reshape(Um,nxc,nyc,nzc);
  Vin = reshape(Vm,nxc,nyc,nzc);
  clear Tm Sm Um Vm
  
  if strcmp(obcsN,'o') == 1
    inorth = find(OB_Jnorth~=0);
    for ii = inorth
      for iproc = 1:nPy
      TobcsN(ii,iproc,:,indt) = Tin(ii,OB_Jnorth(ii),:);
      SobcsN(ii,iproc,:,indt) = Sin(ii,OB_Jnorth(ii),:);
      UobcsN(ii,iproc,:,indt) = Uin(ii,OB_Jnorth(ii),:);
      VobcsN(ii,iproc,:,indt) = Vin(ii,OB_Jnorth(ii),:); 
      end
    end
  end

  if strcmp(obcsS,'o') == 1
    isouth = find(OB_Jsouth~=0);
    for ii = isouth
      for iproc = 1:nPy
      TobcsS(ii,iproc,:,indt) = Tin(ii,OB_Jsouth(ii)  ,:);
      SobcsS(ii,iproc,:,indt) = Sin(ii,OB_Jsouth(ii)  ,:);
      UobcsS(ii,iproc,:,indt) = Uin(ii,OB_Jsouth(ii)  ,:);
      VobcsS(ii,iproc,:,indt) = Vin(ii,OB_Jsouth(ii)+1,:);
      end
    end
  end
    
  if strcmp(obcsE,'o') == 1
    jeast = find(OB_Ieast~=0);
    for jj = jeast
      for iproc = 1:nPx
      TobcsE(jj,iproc,:,indt) = Tin(OB_Ieast(jj),jj,:);
      SobcsE(jj,iproc,:,indt) = Sin(OB_Ieast(jj),jj,:);
      UobcsE(jj,iproc,:,indt) = Uin(OB_Ieast(jj),jj,:);
      VobcsE(jj,iproc,:,indt) = Vin(OB_Ieast(jj),jj,:);    
      end
    end
  end
  
  if strcmp(obcsW,'o') == 1
    jwest = find(OB_Iwest~=0);
    for jj = jwest
      for iproc = 1:nPx
      TobcsW(jj,iproc,:,indt) = Tin(OB_Iwest(jj)  ,jj,:);
      SobcsW(jj,iproc,:,indt) = Sin(OB_Iwest(jj)  ,jj,:);
      UobcsW(jj,iproc,:,indt) = Uin(OB_Iwest(jj)+1,jj,:);
      VobcsW(jj,iproc,:,indt) = Vin(OB_Iwest(jj)  ,jj,:);
      end          
    end
  end
  
  clear Tin Sin Uin Vin
end

%% Save outputs
  
if strcmp(obcsS,'o') == 1
  wrslice([fpath 'Rs_TobcsS_' date '_nPy' num2str(nPy) '.bin' ],TobcsS,1,fmt,Ieee);
  wrslice([fpath 'Rs_SobcsS_' date '_nPy' num2str(nPy) '.bin' ],SobcsS,1,fmt,Ieee);
  wrslice([fpath 'Rs_UobcsS_' date '_nPy' num2str(nPy) '.bin' ],UobcsS,1,fmt,Ieee);
  wrslice([fpath 'Rs_VobcsS_' date '_nPy' num2str(nPy) '.bin' ],VobcsS,1,fmt,Ieee);
end 
  
if strcmp(obcsE,'o') == 1
  wrslice([fpath 'Rs_TobcsE_' date '_nPx' num2str(nPx) '.bin' ],TobcsE,1,fmt,Ieee);
  wrslice([fpath 'Rs_SobcsE_' date '_nPx' num2str(nPx) '.bin' ],SobcsE,1,fmt,Ieee);
  wrslice([fpath 'Rs_UobcsE_' date '_nPx' num2str(nPx) '.bin' ],UobcsE,1,fmt,Ieee);
  wrslice([fpath 'Rs_VobcsE_' date '_nPx' num2str(nPx) '.bin' ],VobcsE,1,fmt,Ieee);
end 
    
if strcmp(obcsW,'o') == 1
  wrslice([fpath 'Rs_TobcsW_' date '_nPx' num2str(nPx) '.bin' ],TobcsW,1,fmt,Ieee);
  wrslice([fpath 'Rs_SobcsW_' date '_nPx' num2str(nPx) '.bin' ],SobcsW,1,fmt,Ieee);
  wrslice([fpath 'Rs_UobcsW_' date '_nPx' num2str(nPx) '.bin' ],UobcsW,1,fmt,Ieee);
  wrslice([fpath 'Rs_VobcsW_' date '_nPx' num2str(nPx) '.bin' ],VobcsW,1,fmt,Ieee);
end 